import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import '../features/call/call_page.dart';
import '../features/call/incoming_call_page.dart';

class CallService {
  static const MethodChannel _channel = MethodChannel('call_service');
  static String? currentPhoneNumber;
  static bool _isCallPageVisible = false;

  static final GlobalKey<NavigatorState> navigatorKey = GlobalKey<NavigatorState>();

  CallService() {
    _channel.setMethodCallHandler((call) async {
      final context = navigatorKey.currentContext;
      if (context == null) {
        print('CallService: Navigator context is null, cannot navigate');
        return;
      }

      switch (call.method) {
        case "callAdded":
          final phoneNumber = call.arguments["callId"] as String;
          final state = call.arguments["state"] as String;
          currentPhoneNumber = phoneNumber.replaceFirst('tel:', '');
          print('CallService: Call added, number: $currentPhoneNumber, state: $state');
          if (state == "ringing") {
            _navigateToIncomingCallPage(context);
          } else {
            _navigateToCallPage(context);
          }
          break;
        case "callStateChanged":
          final state = call.arguments["state"] as String;
          print('CallService: State changed to $state');
          if (state == "disconnected" || state == "disconnecting") {
            _closeCallPage(context);
          } else if (state == "active" || state == "dialing") {
            _navigateToCallPage(context);
          } else if (state == "ringing") {
            _navigateToIncomingCallPage(context);
          }
          break;
        case "callEnded":
        case "callRemoved":
          print('CallService: Call ended/removed');
          _closeCallPage(context);
          currentPhoneNumber = null;
          break;
        case "incomingCallFromNotification":
          final phoneNumber = call.arguments["phoneNumber"] as String;
          currentPhoneNumber = phoneNumber;
          print('CallService: Incoming call from notification: $phoneNumber');
          _navigateToIncomingCallPage(context);
          break;
      }
    });

    // Signal Flutter is ready
    WidgetsFlutterBinding.ensureInitialized();
    _channel.invokeMethod("flutterReady");
  }

  void _navigateToCallPage(BuildContext context) {
    print('CallService: Navigating to CallPage');
    Navigator.push(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/call'),
        builder: (context) => CallPage(
          displayName: currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: null,
        ),
      ),
    ).then((_) {
      _isCallPageVisible = false;
      print('CallService: CallPage popped, _isCallPageVisible set to false');
    });
    _isCallPageVisible = true;
  }

  void _navigateToIncomingCallPage(BuildContext context) {
    print('CallService: Navigating to IncomingCallPage');
    Navigator.push(
      context,
      MaterialPageRoute(
        settings: const RouteSettings(name: '/incoming_call'),
        builder: (context) => IncomingCallPage(
          displayName: currentPhoneNumber!,
          phoneNumber: currentPhoneNumber!,
          thumbnail: null,
        ),
      ),
    ).then((_) {
      _isCallPageVisible = false;
      print('CallService: IncomingCallPage popped, _isCallPageVisible set to false');
    });
    _isCallPageVisible = true;
  }

  void _closeCallPage(BuildContext context) {
    print('CallService: Attempting to close call page, _isCallPageVisible: $_isCallPageVisible');
    if (Navigator.canPop(context)) {
      print('CallService: Popping call page');
      Navigator.pop(context);
      _isCallPageVisible = false;
    } else {
      print('CallService: No page to pop');
    }
  }

  Future<Map<String, dynamic>> makeGsmCall(
    BuildContext context, {
    required String phoneNumber,
    String? displayName,
    Uint8List? thumbnail,
  }) async {
    try {
      currentPhoneNumber = phoneNumber;
      print('CallService: Making GSM call to $phoneNumber');
      final result = await _channel.invokeMethod('makeGsmCall', {"phoneNumber": phoneNumber});
      print('CallService: makeGsmCall result: $result');
      final resultMap = Map<String, dynamic>.from(result as Map);
      if (resultMap["status"] != "calling") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to initiate call")),
        );
      }
      return resultMap;
    } catch (e) {
      print("CallService: Error making call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error making call: $e")),
      );
      rethrow;
    }
  }

  Future<Map<String, dynamic>> hangUpCall(BuildContext context) async {
    try {
      print('CallService: Hanging up call');
      final result = await _channel.invokeMethod('hangUpCall');
      print('CallService: hangUpCall result: $result');
      final resultMap = Map<String, dynamic>.from(result as Map);
      if (resultMap["status"] != "ended") {
        ScaffoldMessenger.of(context).showSnackBar(
          SnackBar(content: Text("Failed to end call")),
        );
      }
      return resultMap;
    } catch (e) {
      print("CallService: Error hanging up call: $e");
      ScaffoldMessenger.of(context).showSnackBar(
        SnackBar(content: Text("Error hanging up call: $e")),
      );
      rethrow;
    }
  }
}